<?php

use S2lowLegacy\Class\helios\HeliosAnalyseFichierRecu;
use S2lowLegacy\Class\S2lowLogger;
use S2lowLegacy\Model\AuthoritySiretSQL;
use S2lowLegacy\Model\AuthoritySQL;
use S2lowLegacy\Model\HeliosRetourSQL;
use S2lowLegacy\Model\HeliosTransactionsSQL;

class HeliosAnalyseFichierRecuTest extends S2lowTestCase
{
    /** @var HeliosDirectoriesManager  */
    private HeliosDirectoriesManager $heliosDirectoriesManager;
    /** @var  HeliosTransactionsSQL */
    private $heliosTransactionSQL;

    public function setUp(): void
    {
        parent::setUp();
        $this->heliosDirectoriesManager = $this->getObjectInstancier()->get(HeliosDirectoriesManager::class);
        $this->heliosDirectoriesManager->createDirectories();
        $this->heliosTransactionSQL = new HeliosTransactionsSQL($this->getSQLQuery());
    }

    public function tearDown(): void
    {
        parent::tearDown(); // TODO: Change the autogenerated stub
        $this->heliosDirectoriesManager->clearDirectories();
    }

    public function testGetAllDirectoryVide()
    {
        $filesInDirectory = $this->getHeliosAnalyseFichierReponse()
                                ->getAllDirectory($this->heliosDirectoriesManager->helios_ftp_response_tmp_local_path);
        $this->assertEquals([], $filesInDirectory);
        $this->assertEquals("Aucun fichier à analyser", $this->getLogRecords()[1]['message']);
    }

    public function testGetAllDirectoryDeuxFichiers()
    {
        file_put_contents(
            $this->heliosDirectoriesManager->helios_ftp_response_tmp_local_path . "/pes_acquit.xml",
            file_get_contents(__DIR__ . "/fixtures/pes_acquit.xml")
        );
        file_put_contents(
            $this->heliosDirectoriesManager->helios_ftp_response_tmp_local_path . "/pes_retour.xml",
            file_get_contents(__DIR__ . "/fixtures/pes_acquit.xml")
        );

        $filesInDirectory = $this->getHeliosAnalyseFichierReponse()
            ->getAllDirectory($this->heliosDirectoriesManager->helios_ftp_response_tmp_local_path);

        $this->assertTrue(in_array("pes_acquit.xml", $filesInDirectory));
        $this->assertTrue(in_array("pes_retour.xml", $filesInDirectory));
        $this->assertEquals(2, count($filesInDirectory));
    }



    private function analyse(string $filename, $response_root = null)
    {
        if (is_null($response_root)) {
            $response_root = $this->heliosDirectoriesManager->helios_response_root;
        }
        $this->getHeliosAnalyseFichierReponse()->analyseOneFileForWorker(
            $this->heliosDirectoriesManager->helios_ftp_response_tmp_local_path,
            $response_root,
            $this->heliosDirectoriesManager->helios_responses_error_path,
            $this->heliosDirectoriesManager->helios_ocre,
            $filename
        );
    }

    private function getHeliosAnalyseFichierReponse()
    {
        $heliosTransactionsSQL = new HeliosTransactionsSQL($this->getSQLQuery());
        $heliosRetourSQL = new HeliosRetourSQL($this->getSQLQuery());
        $authoritySiretSQL = new AuthoritySiretSQL($this->getSQLQuery());
        $schema_pes_path = HELIOS_XSD_PATH;
        $s2lowLogger = $this->getObjectInstancier()->get(S2lowLogger::class);
        return new HeliosAnalyseFichierRecu($heliosTransactionsSQL, $heliosRetourSQL, $authoritySiretSQL, $schema_pes_path, "noreply@sigmalis.com", "noreply@sigmalis.com", $s2lowLogger);
    }

    public function testAnalysePesRetour()
    {
        $authoritySireSQL = new AuthoritySiretSQL($this->getSQLQuery());
        $authoritySireSQL->add(1, "12345678900035");

        $this->analysePesRetour(__DIR__ . "/fixtures/pes_retour.xml");
        $this->assertMatchesRegularExpression(
            '#Traitement de /tmp/phpunit.*/helios_ftp_response_tmp_local_path/pes_retour.xml#i',
            $this->getLogRecords()[0]['message']
        );

        $this->assertTrue(file_exists($this->heliosDirectoriesManager->helios_response_root . "/pes_retour.xml"));
        $heliosRetourSQL = new HeliosRetourSQL($this->getSQLQuery());
        $info = $heliosRetourSQL->getInfoFromFilename(1, "pes_retour.xml");
        $this->assertEquals("12345678900035", $info['siret']);
    }

    public function testAnalysePesRetourErreurRename()
    {
        $authoritySireSQL = new AuthoritySiretSQL($this->getSQLQuery());
        $authoritySireSQL->add(1, "12345678900035");

        $this->analysePesRetour(__DIR__ . "/fixtures/pes_retour.xml", "/Rep/qui/existe/pas");
        //print_r($this->getLogRecords()[4]['formatted']);
        $this->assertMatchesRegularExpression(
            '#Traitement de /tmp/phpunit.*/helios_ftp_response_tmp_local_path/pes_retour.xml annulé : déplacement impossible#i',
            $this->getLogRecords()[3]['formatted']
        );
        $this->assertFalse(file_exists($this->heliosDirectoriesManager->helios_response_root . "/pes_retour.xml"));
        $this->assertTrue(file_exists($this->heliosDirectoriesManager->helios_ftp_response_tmp_local_path . "/pes_retour.xml"));
    }

    private function analysePesRetour($file_path, $response_root = null)
    {
        $filename = basename($file_path);
        copy($file_path, $this->heliosDirectoriesManager->helios_ftp_response_tmp_local_path . "/$filename");
        $this->analyse($filename, $response_root);
    }

    public function testAnalysePesRetourNonAbonne()
    {
        //$this->expectOutputRegex("#La collectivité 66920145100015 n'est pas abonnée à l'application Comptabilité Publique du TdT#");
        $this->analysePesRetour(__DIR__ . "/fixtures/pes_retour_nonabonne.xml");
        $this->assertFalse(file_exists($this->heliosDirectoriesManager->helios_response_root . "/pes_retour_nonabonne.xml"));
        $logs_records = $this->getLogRecords();
        $this->assertEquals(
            "[ERREUR] La collectivité 66920145100015 n'est pas abonnée à l'application Comptabilité Publique du TdT, elle n'est donc pas autorisée à recevoir le PES_Retour ",
            $logs_records[2]['message']
        );
    }

    public function testAnalyseDeplacementErreurImpossible()
    {
        copy(__DIR__ . "/fixtures/pes_retour_nonabonne.xml", $this->heliosDirectoriesManager->helios_responses_error_path . "/pes_retour_nonabonne.xml");

        $this->analysePesRetour(__DIR__ . "/fixtures/pes_retour_nonabonne.xml");
        $this->assertTrue(file_exists($this->heliosDirectoriesManager->helios_responses_error_path . "/pes_retour_nonabonne.xml"));
        $this->assertTrue(file_exists($this->heliosDirectoriesManager->helios_responses_error_path . "/pes_retour_nonabonne.xml.1"));
        $logs_records = $this->getLogRecords();
        $this->assertEquals(
            "[WARNING] Le fichier pes_retour_nonabonne.xml existe déjà dans le répertoire des fichiers en erreur : renommé en *.1",
            $logs_records[3]['message']
        );
    }

    public function testAnalysePesRetourDoubleSire()
    {
        $authoritySireSQL = new AuthoritySiretSQL($this->getSQLQuery());
        $authoritySireSQL->add(1, "12345678900035");
        $authoritySireSQL->add(2, "12345678900035");
        $this->analysePesRetour(__DIR__ . "/fixtures/pes_retour.xml");
        $this->assertFalse(file_exists($this->heliosDirectoriesManager->helios_response_root . "/pes_retour.xml"));
        $logs_records = $this->getLogRecords();
        $this->assertEquals(
            "[ERREUR] Le SIRET 12345678900035 est associé à plusieurs collectivités. Le PES_Retour n'est donc pas attribué",
            $logs_records[2]['message']
        );
    }

    public function testOcre()
    {
        $filename = "toto.ocre";
        file_put_contents($this->heliosDirectoriesManager->helios_ftp_response_tmp_local_path . "/$filename", "test");
        //$this->expectOutputRegex("#Traitement de vfs://test/helios_ftp_response_tmp_local_path/toto.ocre#");
        $this->analyse($filename);
        $this->assertTrue(file_exists($this->heliosDirectoriesManager->helios_ocre . "/" . $filename));
        $logs_records = $this->getLogRecords();
        $this->assertMatchesRegularExpression(
            "#Traitement de /tmp/phpunit.*/helios_ftp_response_tmp_local_path/toto.ocre#i",
            $logs_records[0]['message']
        );
    }

    public function testPesAcquitNotFound()
    {
        $filename = "pes_acquit.xml";
        file_put_contents(
            $this->heliosDirectoriesManager->helios_ftp_response_tmp_local_path . "/$filename",
            file_get_contents(__DIR__ . "/fixtures/pes_acquit.xml")
        );
        $this->analyse($filename);
        $logs_records = $this->getLogRecords();
        $this->assertEquals(
            "[ERREUR] L'identificant NomFic pescg291201703030412001 n'est associé à aucune transaction dans la base de données",
            $logs_records[2]['message']
        );
    }

    public function testPesAcquit()
    {
        $transaction_id = $this->createPESAller();
        $filename = "pes_acquit.xml";
        file_put_contents(
            $this->heliosDirectoriesManager->helios_ftp_response_tmp_local_path . "/$filename",
            file_get_contents(__DIR__ . "/fixtures/pes_acquit.xml")
        );
        $this->analyse($filename);
        $logs_records = $this->getLogRecords();
        $this->assertEquals(
            "Transaction $transaction_id : information disponible",
            $logs_records[3]['message']
        );
        $heliosTransactionSQL = $this->getObjectInstancier()->get(HeliosTransactionsSQL::class);
        $this->assertEquals(
            HeliosTransactionsSQL::INFORMATION_DISPONIBLE,
            $heliosTransactionSQL->getLatestStatusId($transaction_id)
        );
    }

    public function testPesAcquitErreurRename()
    {
        $transaction_id = $this->createPESAller();
        $filename = "pes_acquit.xml";
        file_put_contents(
            $this->heliosDirectoriesManager->helios_ftp_response_tmp_local_path . "/$filename",
            file_get_contents(__DIR__ . "/fixtures/pes_acquit.xml")
        );
        $this->analyse($filename, "/repertoire/Non/Existant");
        $logs_records = $this->getLogRecords();
        $this->assertMatchesRegularExpression(
            "#Traitement.*/helios_ftp_response_tmp_local_path/pes_acquit.xml annulé : déplacement impossible#i",
            $logs_records[5]['message']
        );
        $this->assertFalse(file_exists($this->heliosDirectoriesManager->helios_response_root . "/pes_acquit.xml"));
        $this->assertTrue(file_exists($this->heliosDirectoriesManager->helios_ftp_response_tmp_local_path . "/pes_acquit.xml"));

        $heliosTransactionSQL = $this->getObjectInstancier()->get(HeliosTransactionsSQL::class);
        $this->assertEquals(
            HeliosTransactionsSQL::TRANSMIS,
            $heliosTransactionSQL->getLatestStatusId($transaction_id)
        );
    }

    private function createPESAller()
    {
        $heliosTransactionSQL = new HeliosTransactionsSQL($this->getSQLQuery());
        $transaction_id = $heliosTransactionSQL->create("toto", "xxx", 8, 1, 42, "123");

        $info  = array(
            'nom_fic' => "pescg291201703030412001",
            'cod_col' => 400,
            'cod_bud' => 01,
            'id_post' => "086016",
        );

        $heliosTransactionSQL->setInfoFromPESAller($transaction_id, $info);
        $heliosTransactionSQL->updateStatus($transaction_id, HeliosTransactionsSQL::TRANSMIS, "test");
        return $transaction_id;
    }

    private function recupPESAcquit($expectedLastLogMessage)
    {
        $filename = "pes_acquit.xml";
        file_put_contents(
            $this->heliosDirectoriesManager->helios_ftp_response_tmp_local_path . "/$filename",
            file_get_contents(__DIR__ . "/fixtures/pes_acquit.xml")
        );
        $this->analyse($filename);
        $logs_records = $this->getLogRecords();
        $this->assertEquals($expectedLastLogMessage, $logs_records[3]['message']);
    }

    /**
     * @throws Exception
     */
    public function testPesAcquitDeuxPES()
    {
        $transaction_id_1 = $this->createPESAller();
        $transaction_id_2 = $this->createPESAller();

        $sql = "UPDATE helios_transactions_workflow SET date=? WHERE transaction_id=?";
        $this->getSQLQuery()->query($sql, "1970-01-01", $transaction_id_1);

        $this->recupPESAcquit("Transaction {$transaction_id_2} : information disponible");
    }

    /**
     * @throws Exception
     */
    public function testPesAcquitDeuxPESBefore()
    {
        $transaction_id_1 = $this->createPESAller();
        $transaction_id_2 = $this->createPESAller();

        $sql = "UPDATE helios_transactions_workflow SET date=? WHERE transaction_id=?";
        $this->getSQLQuery()->query($sql, "1970-01-01", $transaction_id_2);

        $this->recupPESAcquit("Transaction {$transaction_id_1} : information disponible");
    }

    public function testPesAcquitDeuxPESUnTransmis()
    {
        $transaction_id_1 = $this->createPESAller();
        $transaction_id_2 = $this->createPESAller();
        $this->heliosTransactionSQL->updateStatus(
            $transaction_id_1,
            HeliosTransactionsSQL::ACQUITTER,
            "test"
        );
        $this->recupPESAcquit("Transaction {$transaction_id_2} : information disponible");
    }

    public function testPesAcquitDeuxPESUnTransmisDeux()
    {
        $transaction_id_1 = $this->createPESAller();
        $transaction_id_2 = $this->createPESAller();
        $this->heliosTransactionSQL->updateStatus(
            $transaction_id_2,
            HeliosTransactionsSQL::ACQUITTER,
            "test"
        );

        $this->recupPESAcquit("Transaction {$transaction_id_1} : information disponible");
    }

    /**
     * @throws Exception
     */
    public function testMalformedAcquit()
    {
        $transaction_id = $this->createPESAller();
        $filename = "pes_acquit_not_valid.xml";
        file_put_contents(
            $this->heliosDirectoriesManager->helios_ftp_response_tmp_local_path . "/$filename",
            file_get_contents(__DIR__ . "/fixtures/pes_acquit_not_valid.xml")
        );

        $this->getHeliosAnalyseFichierReponse()->analyseOneFile(
            $this->heliosDirectoriesManager->helios_ftp_response_tmp_local_path . "/$filename",
            $this->heliosDirectoriesManager->helios_response_root,
            $this->heliosDirectoriesManager->helios_ocre,
            HELIOS_XSD_PATH
        );
        $logs_records = $this->getLogRecords();
        $this->assertEquals(
            "Transaction $transaction_id : erreur retournée par Helios",
            $logs_records[4]['message']
        );
    }

    /**
     * @throws Exception
     */
    public function testMalformedAcquitErreurRename()
    {
        $transaction_id = $this->createPESAller();
        $filename = "pes_acquit_not_valid.xml";
        file_put_contents(
            $this->heliosDirectoriesManager->helios_ftp_response_tmp_local_path . "/$filename",
            file_get_contents(__DIR__ . "/fixtures/pes_acquit_not_valid.xml")
        );

        $this->getHeliosAnalyseFichierReponse()->analyseOneFile(
            $this->heliosDirectoriesManager->helios_ftp_response_tmp_local_path . "/$filename",
            "/Un/Rep/Inexistant",
            $this->heliosDirectoriesManager->helios_ocre,
            HELIOS_XSD_PATH
        );
        $logs_records = $this->getLogRecords();
        $this->assertMatchesRegularExpression(
            "#Trai.*/helios_ftp_response_tmp_local_path//pes_acquit_not_valid.xml annulé : déplacement impossible#i",
            $logs_records[6]['message']
        );

        $heliosTransactionSQL = $this->getObjectInstancier()->get(HeliosTransactionsSQL::class);
        $this->assertEquals(
            HeliosTransactionsSQL::TRANSMIS,
            $heliosTransactionSQL->getLatestStatusId($transaction_id)
        );
    }

    public function testMalformaedAcquitWithoutCodCol()
    {
        $transaction_id = $this->createPESAller();
        $filename = "pes_acquit_not_valid_without_cod_col.xml";
        file_put_contents(
            $this->heliosDirectoriesManager->helios_ftp_response_tmp_local_path . "/$filename",
            file_get_contents(__DIR__ . "/fixtures/pes_acquit_not_valid_without_cod_col.xml")
        );
        $this->getHeliosAnalyseFichierReponse()->analyseOneFile(
            $this->heliosDirectoriesManager->helios_ftp_response_tmp_local_path . "/$filename",
            $this->heliosDirectoriesManager->helios_response_root,
            $this->heliosDirectoriesManager->helios_ocre,
            HELIOS_XSD_PATH
        );
        $logs_records = $this->getLogRecords();
        $this->assertEquals(
            "Transaction $transaction_id : erreur retournée par Helios",
            $logs_records[4]['message']
        );
    }

    public function testBadRenamePesAcquit()
    {
        $heliosTransactionSQL = $this->getObjectInstancier()->get(HeliosTransactionsSQL::class);
        $transaction_id = $heliosTransactionSQL->create(
            "toto",
            "xxx",
            8,
            1,
            42,
            "123"
        );

        $info = array(
            'nom_fic' => "pescg291201703030412001",
            'cod_col' => 400,
            'cod_bud' => 01,
            'id_post' => "086016",
        );

        $heliosTransactionSQL->setInfoFromPESAller($transaction_id, $info);
        $heliosTransactionSQL->updateStatus($transaction_id, HeliosTransactionsSQL::TRANSMIS, "test");

        $filename = "pes_acquit.xml";
        file_put_contents(
            $this->heliosDirectoriesManager->helios_ftp_response_tmp_local_path . "/$filename",
            file_get_contents(__DIR__ . "/fixtures/pes_acquit.xml")
        );
        $this->analyse($filename, "Nim/Por/Te/Quoi");

        $this->assertEquals(
            HeliosTransactionsSQL::TRANSMIS,
            $heliosTransactionSQL->getLatestStatusId($transaction_id)
        );
    }
}
